/**
 * @summary     concatenateSeq
 * @description Concatenate 2 or more sequences and validate this concatenation.
 * @version     
 * @file        concatenateSeq.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/** Concatenate node sequence, when it's possible, in order of the user click on graph nodes.
 * @module concatSelectedSeq
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery 1.6.4</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @requires gritter
 * @requires Biojs
 * @requires Biojs.Sequence
 * @requires Biojs.Sequence.extend
 * 
 * @example
 *	//Concatenate 2 or more sequences nodes
 *	concatenateSeq.init([list of ordering nodes]);
 *  
 *	//Validate the concatenation
 *	concatenateSeq.validate(node, targetNode, kmerLen, edgeDirection);
 * 
 *
 */

/** 
 * Define an object with complement and reverse methods come from Biojs.Sequence.extend.
 * This object have methods to concatenate sequences contained in nodes of a graph only if good way has been chosen :
 * the order of click define the way.
 * If a way don't allow concatenation a message in a bubble is display.
 * 
 * @class concatSelectedSeq
 * @constructor
 *
 */
	"use strict";
	var concatenateSeq = function() {//DUMMY
	
	};
	
	/**
	* Transform a sequence in the complement of this sequence.
	* @method complement
	* @param {String} sequence The target sequence
	* @return {String} The complement of the sequence
	* @private
	*/
	concatenateSeq.complement = Biojs.Sequence.prototype.complement;
		
	/**
	 * Transform a sequence in the reverse of this sequence.
	 * @method reverse
	 * @param {String} sequence The target sequence
	 * @return {String} The reverse of the sequence
	 * @private
	 *  
	 */
	concatenateSeq.reverse = Biojs.Sequence.prototype.reverse;
	
	/**
	 * Concatenate node sequence, when it's possible, in order of the user click.
	 * @method init
	 * @param {array} collection List of nodes in order of click ( a node is a cytoscape node object)
	 * @return {String} concatenate nodes sequences.
	 */
	concatenateSeq.init = function(collection) {
		var concatSequence = "", 
			concatHeader = "", sequenceString = [], 
			kmerLen = collection[0].data("k"), 
			collectionLen = collection.length;
	
		this.selectedEl = collection;
		if (collectionLen <= 1) {//test if collection contained only one element and alert and stop function if it's the case
			$.extend($.gritter.options, {
				position : 'bottom-right',
	
			});
			$.gritter.add({
				title : 'Concatenate warning',
				text : "Please select two nodes or more.",
				sticky : false,
				time : 8000,
				class_name : 'alert-gritter',
			});
			
			return false;
		}
	
		cy.edges(":selected").unselect();
		/* define condition concatenation, alert and break loop */
		var sequencePrecedente, 
			isAlerted = false, 
			leaveConditionFF, leaveConditionFR, leaveConditionRF,leaveConditionRR, enterConditionRR, 
			enterConditionFF, enterConditionFR, enterConditionRF;
	
		/* first loop on nodes selected */
		for (var n = 0; n < collectionLen; n++) {
	
			var thisId = collection[n].data("id"), nbConnectedEdges = collection[n]._private.edges.length, isTarget;
	
			/* second loop on all edges of nodes selected */
			for (var h = 0; h < nbConnectedEdges; h++) {
				var edge = collection[n]._private.edges[h], 
					target = cy.nodes("#" + edge.data("target")), 
					source = cy.nodes("#" + edge.data("source")), 
					targetId = target.data("id"), 
					sourceId = source.data("id"), 
					directionEdgesLeave, directionEdgesEnter;
	
				/*using target nodes for concatenate sequences so the way is between 0 and n-1.
				 To concatenate, it's necessary to have target of edge selected and
				 the target nodes different of the current nodes (repetition of nodes with the 2 loops)*/
				if (n < collectionLen - 1 && targetId != thisId && target.selected()) {
	
					if (targetId == collection[n + 1].data("id")) {//target must be the second one in selected nodes
						/*define edges enter direction and edges leave direction*/
						if (n == 0) {
							/*first node haven't enter condition*/
							enterConditionFF = undefined;
							enterConditionFR = undefined;
							enterConditionRF = undefined;
							enterConditionRR = undefined;
	
							directionEdgesLeave = edge.data("direction");
	
							leaveConditionFF = (directionEdgesLeave == "FF" || directionEdgesLeave == "ff" );
							leaveConditionFR = (directionEdgesLeave == "FR" || directionEdgesLeave == "fr");
							leaveConditionRF = (directionEdgesLeave == "RF" || directionEdgesLeave == "rf");
							leaveConditionRR = (directionEdgesLeave == "RR" || directionEdgesLeave == "rr");
	
							isTarget = targetId;
	
						} else if (isTarget == thisId) {
							/*if the current node is the target of precedent node selected,
							 the enter condition of the current nodes is the leave condition of the precedent node*/
							enterConditionFF = leaveConditionFF;
							enterConditionFR = leaveConditionFR;
							enterConditionRF = leaveConditionRF;
							enterConditionRR = leaveConditionRR;
	
							directionEdgesLeave = edge.data("direction");
	
							leaveConditionFF = (directionEdgesLeave == "FF" || directionEdgesLeave == "ff" );
							leaveConditionFR = (directionEdgesLeave == "FR" || directionEdgesLeave == "fr");
							leaveConditionRF = (directionEdgesLeave == "RF" || directionEdgesLeave == "rf");
							leaveConditionRR = (directionEdgesLeave == "RR" || directionEdgesLeave == "rr");
	
							isTarget = targetId;
						} else {
							break;//break edge loop if the current node is not the target
							
						}
					} else {
						continue;	//if the target of edge is not the next node selected continue with an other edge
	
					
					}
	
					var thisSequence = collection[n].data("sequence"),
						targetSequence = target.data("sequence"),
						testConcatSeq, initialDirection;
	
					if (n == 0) {//concatenation for the first node with the second
						concatHeader += thisId;
						if (leaveConditionFF) {//concatenate forward with forward
							concatHeader += "-" + targetId;
							sequenceString[0] = thisSequence;
							sequenceString[1] = targetSequence.substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							initialDirection = "FF";
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if (leaveConditionRF) {//concatenate reverse with forward
							concatHeader += "-" + targetId;
							sequenceString[0] = this.reverse(this.complement(thisSequence));
							sequenceString[1] = targetSequence.substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							initialDirection = "RF";
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if (leaveConditionRR) {//concatenate reverse with reverse
							concatHeader += "-" + targetId;
							sequenceString[0] = this.reverse(this.complement(thisSequence));
							sequenceString[1] = this.reverse(this.complement(targetSequence)).substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							initialDirection = "RR";
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if (leaveConditionFR) {//concatenate forward with reverse
							concatHeader += "-" + targetId;
							sequenceString[0] = thisSequence;
							sequenceString[1] = this.reverse(this.complement(targetSequence)).substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							initialDirection = "FR";
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if (!isAlerted) {//alert gritter if enter and leave condition are not respected
							$.extend($.gritter.options, {
								position : 'bottom-right',
	
							});
							$.gritter.add({
								title : 'Concatenate warning',
								text : "Node " + thisId.bold() + " can't be concatenated with node " + target.data("id").bold() + ".",
								sticky : false,
								time : 8000,
								class_name : 'alert-gritter',
							});
							isAlerted = true;
							break;
							
						}
	
					/*Concatenation for others nodes*/
					} else if (n < collectionLen - 1) {
						if (enterConditionFF && leaveConditionFF) {//concatenate forward with forward
							concatHeader += "-" + targetId;
							if (concatSequence == "") {
								sequenceString[0] = thisSequence;
	
							} else {
								sequenceString[0] = concatSequence;
							}
	
							sequenceString[1] = targetSequence.substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if (enterConditionRR && leaveConditionRR) {//concatenate reverse with reverse
							concatHeader += "-" + targetId;
							if (concatSequence == "") {
								sequenceString[0] = this.reverse(this.complement(thisSequence));
	
							} else {
								sequenceString[0] = this.reverse(this.complement(concatSequence));
							}
	
							sequenceString[1] = this.reverse(this.complement(targetSequence)).substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if ((enterConditionFF && leaveConditionFR) || (enterConditionFR && leaveConditionRR)) {//concatenate forward with reverse
							concatHeader += "-" + targetId;
							if (concatSequence == "") {
								sequenceString[0] = thisSequence;
	
							} else {
								sequenceString[0] = concatSequence;
							}
	
							sequenceString[1] = this.reverse(this.complement(targetSequence)).substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if ((enterConditionRR && leaveConditionRF) || (enterConditionRF && leaveConditionFF)) {//concatenate reverse with forward
							concatHeader += "-" + targetId;
							if (concatSequence == "") {
								sequenceString[0] = this.reverse(this.complement(thisSequence));
	
							} else {
								sequenceString[0] = this.reverse(this.complement(concatSequence));
							}
	
							sequenceString[1] = targetSequence.substring(kmerLen - 1);
							concatSequence = sequenceString[0] + sequenceString[1];
							testConcatSeq = this.validate(collection[n], target, kmerLen, edge.data("direction"));
							edge.select();
							if (!testConcatSeq) {//Alert if concatenation is not validate
								alert("Error with concatenate function: k-mer of " + thisId.bold() + " and " + targetId.bold() + " are not identical");
							};
	
							//console.log(thisId, target.data("id"), concatSequence, edge.data("direction"))
	
						} else if (!isAlerted) {//alert gritter if enter and leave condition are not respected
	
							$.extend($.gritter.options, {
								position : 'bottom-right',
	
							});
							$.gritter.add({
								title : 'Concatenate warning',
								text : "Node " + thisId.bold() + " can't be concatenated with node " + target.data("id").bold() + ".",
								sticky : false,
								time : 8000,
								class_name : 'alert-gritter',
							});
							isAlerted = true;
							break;
	
						}
					}
	
				}
	
			}//end of second loop for edges
	
			/*Alert if the sequence concatenate haven't changed compare to the precedent lap of nodes selected loop
			 alert only when:
			 - target is different of the current node ( 2 loops cause repetition),
			 - the is not the first or the last
			 - the target of the edge is selected
			 - the target of the edge is not in selected nodes*/
			if (concatSequence == sequencePrecedente && n > 0 && n < collectionLen - 1 && target.selected() && isTarget != undefined && isTarget != collection[n + 1].data("id") && !isAlerted) {
				isAlerted = true, $.extend($.gritter.options, {
					position : 'bottom-right',
	
				});
				$.gritter.add({
					title : 'Concatenate warning',
					text : "Node " + isTarget.bold() + " can't be concatenated with node " + collection[n + 1].data("id").bold() + ".",
					sticky : false,
					time : 8000,
					class_name : 'alert-gritter',
				});
				isAlerted = true;
				break;
	
			} else {
				sequencePrecedente = concatSequence;
			}
	
		}//end of first loop for nodes selected
	
		/*Show concatenate sequences in south panel*/
		if (concatSequence != "" && concatSequence != undefined && !isAlerted) {
			var $onSequenceSelectionChanged = $("#onSequenceSelectionChanged"), 
				formatSelected = $("#setFormat > ul").find(".ui-menu-checked").siblings("label").text(), 
				sequenceWidth = parseInt($("#paneSouth").width() / 8.2);
	
			/* Add header */
			$(".ui-sequences-selected").after('<div id="divSequence_' + concatHeader + '" class="ui-sequences ui-concatenated-sequences ui-sequences-visible"> </div>');
	
			/* biojs sequence */
			var biojs = new Biojs.Sequence({
				sequence : concatSequence,
				coverage : undefined,
				target : "divSequence_" + concatHeader,
				format : formatSelected,
				id : concatHeader,
				fontSize : '18px',
				columns : {
					size : sequenceWidth
				}
			});
		
			biojs.onSelectionChange(function(e) {	//Display position of selection in sequence node
				$(".ui-sequences-selected").show();
				$onSequenceSelectionChanged.text("In node " + concatHeader + " interval selected: " + e.start + ", " + e.end);
				$onSequenceSelectionChanged.data("selectedSequence", [e.start, e.end]);
				$onSequenceSelectionChanged.data( "id", concatHeader);
			    $onSequenceSelectionChanged.data( "selectedSequence", [e.start, e.end]);
	
			});
	
			$("#divSequence_" + concatHeader).data("biojsData", biojs);
	
			return concatSequence;
	
		} else if (!isAlerted) {//alert gritter if concatSequence is empty or undefined
	
			$.extend($.gritter.options, {
				position : 'bottom-right',
	
			});
			$.gritter.add({
				title : 'Concatenate warning',
				text : "Node " + collection[0].data("id").bold() + " can't be concatenated with node " + collection[1].data("id").bold() + ".",
				time : 8000,
				class_name : 'alert-gritter',
			});
			isAlerted = true;
	
			return false;
		}
	
	};
	
	/**
	 * Check if concatenate sequence is valid between two nodes
	 *
	 * @method   validate
	 * @param {object} node the current node
	 * @param {object} target the target of node
	 * @param {int} kmerLen the length of kmer for sequences in the graph
	 * @return {boolean} return true if concatenation is validate or false if is not
	 */
	concatenateSeq.validate = Biojs.Sequence.prototype.validateLink;
