;(function($){

 var layout_options = {};
    layout_options["forceDirected"] = [
        { id: "gravitation", label: "Gravitation",       value: -500,   tip: "The gravitational constant. Negative values produce a repulsive force." },
        { id: "mass",        label: "Node mass",         value: 3,      tip: "The default mass value for nodes." },
        { id: "tension",     label: "Edge tension",      value: 0.1,    tip: "The default spring tension for edges." },
        { id: "restLength",  label: "Edge rest length",  value: "auto", tip: "The default spring rest length for edges." },
        { id: "drag",        label: "Drag co-efficient", value: 0.4,    tip: "The co-efficient for frictional drag forces." },
        { id: "minDistance", label: "Minimum distance",  value: 1,      tip: "The minimum effective distance over which forces are exerted." },
        { id: "maxDistance", label: "Maximum distance",  value: 10000,  tip: "The maximum distance over which forces are exerted." },
        { id: "weightAttr",  label: "Weight Attribute",  value: "",  tip: "The name of the edge attribute that contains the weights." },
        { id: "weightNorm",  label: "Weight Normalization", value: ["linear","invlinear","log"],  tip: "How to interpret weight values." },
        { id: "seed",        label: "Seed",       value: 0,      tip: "Seed for initial node positions." },
        { id: "iterations",  label: "Iterations",        value: 400,    tip: "The number of iterations to run the simulation." },
        { id: "maxTime",     label: "Maximum time",      value: 30000,  tip: "The maximum time to run the simulation, in milliseconds." },
        { id: "autoStabilize", label: "Auto stabilize",  value: true,   tip: "If checked, Cytoscape Web automatically tries to stabilize results that seems unstable after running the regular iterations." }
    ];
    layout_options["circle"] = [
        { id: "angleWidth", label: "Angle width",    value: 360,   tip: "The angular width of the layout, in degrees." },
        { id: "breadthfirst",       label: "breadthfirst structure", value: false, tip: "Flag indicating if any breadthfirst-structure in the data should be used to inform the layout." }
    ];
    layout_options["radial"] = [
        { id: "radius",     label: "Radius",      value: "auto", tip: "The radius increment between depth levels." },
        { id: "angleWidth", label: "Angle width", value: 360,    tip: "The angular width of the layout, in degrees." }
    ];
    layout_options["breadthfirst"] = [
        { id: "orientation",  label: "Orientation",   value: ["topToBottom","bottomToTop","leftToRight","rightToLeft"], tip: "The orientation of the layout." },
        { id: "depthSpace",   label: "Depth space",   value: 50, tip: "The space between depth levels in the breadthfirst." },
        { id: "breadthSpace", label: "Breadth space", value: 30, tip: "The space between siblings in the breadthfirst." },
        { id: "subbreadthfirstSpace", label: "Subbreadthfirst space",   value: 5,  tip: "The space between different sub-breadthfirsts." }
    ];
    layout_options["compoundSpringEmbedder"] = [
		{ id: "gravitation",                    label: "Gravitation",                       value: -50,   tip: "The gravitational constant. Negative values produce a repulsive force." },
		{ id: "centralGravitation",             label: "Central gravitation",               value: 50,    tip: "All nodes are assumed to be pulled slightly towards the center of the network by a central gravitational force (gravitational constant) during layout." },
		{ id: "centralGravityDistance",         label: "Central gravity distance",          value: 50,    tip: "The radius of the region in the center of the drawing, in which central gravitation is not exerted." },
		{ id: "compoundCentralGravitation",     label: "Compound central gravitation",      value: 50,    tip: " The central gravitational constant for compound nodes." },
		{ id: "compoundCentralGravityDistance", label: "Compound central gravity distance", value: 50,    tip: " The central gravitational constant for compound nodes. " },
		{ id: "tension",              label: "Edge tension",            value: 50,     tip: "The default spring tension for edges." },
		{ id: "restLength",           label: "Edge rest length",        value: 50,     tip: "The default spring rest length for edges." },
		{ id: "smartRestLength",      label: "Smart rest length",       value: true,   tip: "Whether or not smart calculation of ideal rest length should be performed for inter-graph edges." },
		{ id: "layoutQuality",        label: "Layout quality",          value: ["default","draft","proof"], tip: "A better quality layout requires more iterations, taking longer." },
		{ id: "incremental",          label: "Incremental",             value: false,      tip: " If true, layout is applied incrementally by taking current positions of nodes into account." },
		{ id: "uniformLeafNodeSizes", label: "Uniform leaf node sizes", value: false,  tip: " If true, leaf (non-compound or simple) node dimensions are assumed to be uniform, resulting in faster layout." },
		{ id: "smartDistance",        label: "Smart distance",          value: true,  tip: "If true, gravitational repulsion forces are calculated only when node pairs are in a certain range, resulting in faster layout at the relatively minimal cost of layout quality." },
		{ id: "multiLevelScaling",    label: "Multi level scaling",     value: false,  tip: " If true, multi-level scaling algorithm is applied both to better capture the overall structure of the network and to save time on large networks." }
	];
// [settings] Layout settings
    ////////////////////////////////////////////////////////////////////////////////////////////////
   
     create_settings_panel= function (layout_id){

    	var opt = layout_options[layout_id];
    	var panel = ('<div id="'+layout_id+'" class="content ui-widget-content"><table>');

    	for(var i in opt) {
    		var o = opt[i];
    		var v = o.value;
    		panel += '<tr title="'+o.tip+'">';

    		if (typeof v === "function") { v = v(); }

    		if (typeof v === "object") {
    			panel += ('<td align="right"><label>'+o.label+' </label></td><td><select id="'+o.id+'" size="1">');
    			for(var j in v) { panel += ('<option value="'+v[j]+'">'+v[j]+'</option>'); }
    			panel += '</select></td>'
    		} else if (typeof v === "boolean") {
    			panel += ('<td align="right"><label>'+o.label+' </label></td><td align="left"><input type="checkbox" id="'+o.id+'" value="'+v+'"'+(v?' checked="checked"':'')+'/></td>');
    		} else {
    			panel += ('<td align="right"><label>'+o.label+' </label></td><td><input type="text" id="'+o.id+'" value="'+v+'"/></td>');
    		}
    		
    		panel += "</tr>";
    	}
    	
    	panel += "</table></div>";
    	
    	return $(panel);
    }
    
   open_settings = function (layout_names){
	    if ($("#settings").length === 0) {
			var dialog = '\
				<div id="settings" title="Layout Settings">\
					<div class="tabs ui-widget">\
						<ul></ul>\
						<div class="ui-layout-content"/>\
					</div>\
					<div class="footer"><input type="button" id="execute_layout" class="ui-state-default" value="Execute Layout"/></div>\
				</div>\
				';

			$("body").append(dialog);
				
			for(var i in layout_names) {
	            var layout_id = i;
	            var layout_name = layout_names[layout_id];
	            var panel = create_settings_panel(layout_id);
	            $("#settings .tabs ul").append('<li><a href="#'+layout_id+'">' + layout_name + '</a></li>');
	            $("#settings .tabs > .ui-layout-content").append(panel);
	        }
				
			$("#settings .tabs").tabs().addClass('ui-tabs-vertical ui-helper-clearfix');
			$("#settings .tabs li").removeClass('ui-corner-top').addClass('ui-corner-left');
			
			$("#execute_layout").click(function(){
				var layout_id = $("#settings .ui-tabs-selected a").attr("href").replace("#", "");
				var options = {};
				var def = layout_options[layout_id];
				
				for(var i in def) {
					var o = def[i];
		    		var v, id = o.id;
		    		var input = $("#settings #"+layout_id+" #"+id);
		    		v = input.val();
		    		if (input.attr("type") === "checkbox") { v = input.is(":checked"); }
		    		else if (typeof o.value === "number") { v = Number(v); }
		    		options[id] = v;
		    	}
				vis.layout({ name: layout_id, options: options });
			});
			
			$("#settings").dialog({ autoOpen: false, resizable: false, width: 450 });
		}
	    $("#settings").dialog("open");
    }

})(jQuery);
